/*
  ==============================================================================

    LoopbackRecording.h
    Created: 17 Dec 2020 10:29:16am
    Author:  JP music

  ==============================================================================
*/

#pragma once

#include <JuceHeader.h>

#include "jp_utilities.h"

#include "miniaudio.h"


struct CallbackData
{
    jp::MyFifo fifo;
    AudioBuffer<float> buffer{ 2, 2048 };
};

class LoopbackRecorder : public TimeSliceClient
{
public:
    LoopbackRecorder();
    ~LoopbackRecorder();

    void initialize(int deviceIndex = -1, uint32 sampleRate = 0);
    void releaseDevice();

    void readFromFifo(AudioBuffer<float>* buffer, int numOfSamples) { dataBuffer.fifo.readFromFifo(buffer, numOfSamples); }
    int getNumberOfSamplesReady() { return dataBuffer.fifo.abstractFifo.getNumReady(); }
    const StringArray& getAvailableDeviceNames() { return deviceNames; };
    int getDefaultDeviceIndex() { return defaultDeviceIndex; };

    int useTimeSlice() override;


private:
    ma_context context;
    ma_device_info* pPlaybackInfos;
    ma_uint32 playbackCount;
    StringArray deviceNames;

    ma_result result;
    ma_device_config deviceConfig;
    ma_device device;

    ma_backend backends[1];

    CallbackData dataBuffer;

    std::atomic<bool> bStopDevice = false;
    TimeSliceThread timeSliceThread{"DeviceForceStop"};

    int defaultDeviceIndex = -1;
};